;;;; Interface for caching ASDF
;;;;
;;;; This software is part of CLPM. See README.org for more information. See
;;;; LICENSE for license information.

(uiop:define-package #:clpm/asdf
    (:use #:cl
          #:clpm/cache)
  (:export #:asdf-installed-pathname
           #:install-asdf-to-cache))

(in-package #:clpm/asdf)

(defparameter *asdf.lisp*
  (alexandria:read-file-into-string (asdf:system-relative-pathname :asdf "build/asdf.lisp")))

(defun asdf-installed-pathname ()
  "Return the pathname where we would expect to find asdf.lisp installed in
CLPM's cache."
  (clpm-cache-pathname `("asdf" ,(asdf:asdf-version) "asdf.lisp")))

(defun install-asdf-to-cache (&key forcep)
  (let ((pn (asdf-installed-pathname)))
    (ensure-directories-exist pn)
    (with-open-file (s pn :direction :output :if-exists (if forcep :supersede nil))
      (unless (null s)
        (write-string *asdf.lisp* s)))))
